package kz.arta.test.blocking.mysql;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.Executor;

/**
 * Created by exile
 * Date: 18.06.15
 * Time: 13:51
 * класс обертка для соединения, который собирает все открытые стейтменты и закрывает их
 */
public class ConnectionImpl implements Connection {
    private static final Logger LOGGER = LoggerFactory.getLogger(ArtaMysqlConnectionPool.class);

    private Connection connection;
    private List<Statement> statements = new ArrayList<>();

    public ConnectionImpl(Connection connection) {
        this.connection = connection;
    }

    @Override
    public void close() throws SQLException {
        for (Statement st : statements) {
            try {
                st.close();
            } catch (SQLException e) {
                LOGGER.error(e.getMessage(), e);
            }
        }
        connection.close();
        statements = null;
        connection = null;
    }

    private Statement statementOpened(Statement st){
        statements.add(st);
        return new StatementImpl(st, this);
    }

    private PreparedStatement statementOpened(PreparedStatement st) {
        statements.add(st);
        return new PreparedStatementImpl(st, this);
    }

    private CallableStatement statementOpened(CallableStatement st) {
        statements.add(st);
        return new CallableStatementImpl(st, this);
    }

    public void setAutoCommit(boolean autoCommit) throws SQLException {
        connection.setAutoCommit(autoCommit);
    }


    public boolean getAutoCommit() throws SQLException {
        return connection.getAutoCommit();
    }


    public PreparedStatement prepareStatement(String sql) throws SQLException {
        PreparedStatement st = connection.prepareStatement(sql);
        return statementOpened(st);
    }


    public Statement createStatement() throws SQLException {
        Statement st = connection.createStatement();
        return statementOpened(st);
    }


    public void commit() throws SQLException {
        if (!connection.getAutoCommit()) {
            connection.commit();
        }
    }


    public void rollback() throws SQLException {
        connection.rollback();
    }


    public boolean isClosed() throws SQLException {
        return connection.isClosed();
    }


    public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
        Statement st = connection.createStatement(resultSetType, resultSetConcurrency);
        return statementOpened(st);
    }


    public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
        PreparedStatement st = connection.prepareStatement(sql, resultSetType, resultSetConcurrency);
        return statementOpened(st);
    }


    public Savepoint setSavepoint(String name) throws SQLException {
        return connection.setSavepoint(name);
    }

    public Savepoint setSavepoint() throws SQLException {
        return connection.setSavepoint();
    }

    public void rollback(Savepoint savepoint) throws SQLException {
        connection.rollback(savepoint);
    }

    public void releaseSavepoint(Savepoint savepoint) throws SQLException {
        connection.releaseSavepoint(savepoint);
    }

    public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        PreparedStatement st = connection.prepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
        return statementOpened(st);
    }

    public Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        Statement st = connection.createStatement(resultSetType, resultSetConcurrency, resultSetHoldability);
        return statementOpened(st);
    }

    public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException {
        PreparedStatement st = connection.prepareStatement(sql, autoGeneratedKeys);
        return statementOpened(st);
    }

    public PreparedStatement prepareStatement(String sql, String columnNames[]) throws SQLException {
        PreparedStatement st = connection.prepareStatement(sql, columnNames);
        return statementOpened(st);
    }

    public PreparedStatement prepareStatement(String sql, int columnIndexes[]) throws SQLException {
        PreparedStatement st = connection.prepareStatement(sql, columnIndexes);
        return statementOpened(st);
    }


    public CallableStatement prepareCall(String sql) throws SQLException {
        return connection.prepareCall(sql);
    }

    public String nativeSQL(String sql) throws SQLException {
        return connection.nativeSQL(sql);
    }

    public DatabaseMetaData getMetaData() throws SQLException {
        return connection.getMetaData();
    }

    public void setReadOnly(boolean readOnly) throws SQLException {
        connection.setReadOnly(readOnly);
    }

    public boolean isReadOnly() throws SQLException {
        return connection.isReadOnly();
    }

    public void setCatalog(String catalog) throws SQLException {
        connection.setCatalog(catalog);
    }

    public String getCatalog() throws SQLException {
        return connection.getCatalog();
    }

    public void setTransactionIsolation(int level) throws SQLException {
        connection.setTransactionIsolation(level);
    }

    public int getTransactionIsolation() throws SQLException {
        return connection.getTransactionIsolation();
    }

    public SQLWarning getWarnings() throws SQLException {
        return connection.getWarnings();
    }

    public void clearWarnings() throws SQLException {
        connection.clearWarnings();
    }

    public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
        return connection.prepareCall(sql, resultSetType, resultSetConcurrency);
    }

    public Map<String, Class<?>> getTypeMap() throws SQLException {
        return connection.getTypeMap();
    }

    public void setTypeMap(Map<String, Class<?>> map) throws SQLException {
        connection.setTypeMap(map);
    }

    public void setHoldability(int holdability) throws SQLException {
        connection.setHoldability(holdability);
    }

    public int getHoldability() throws SQLException {
        return connection.getHoldability();
    }

    public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency, int resultSetHoldability) throws SQLException {
        CallableStatement st = connection.prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
        return statementOpened(st);
    }

    public Array createArrayOf(String typeName, Object[] elements) throws SQLException {
        return connection.createArrayOf(typeName, elements);
    }

    public Clob createClob() throws SQLException {
        return connection.createClob();
    }

    public Blob createBlob() throws SQLException {
        return connection.createBlob();
    }

    public NClob createNClob() throws SQLException {
        return connection.createNClob();
    }

    public SQLXML createSQLXML() throws SQLException {
        return connection.createSQLXML();
    }

    public boolean isValid(int timeout) throws SQLException {
        return connection.isValid(timeout);
    }

    public void setClientInfo(String name, String value) throws SQLClientInfoException {
        connection.setClientInfo(name, value);
    }

    public void setClientInfo(Properties properties) throws SQLClientInfoException {
        connection.setClientInfo(properties);
    }

    public String getClientInfo(String name) throws SQLException {
        return connection.getClientInfo(name);
    }

    public Properties getClientInfo() throws SQLException {
        return connection.getClientInfo();
    }

    public Struct createStruct(String typeName, Object[] attributes) throws SQLException {
        return connection.createStruct(typeName, attributes);
    }

    public <T> T unwrap(Class<T> iFace) throws SQLException {
        return connection.unwrap(iFace);
    }

    public boolean isWrapperFor(Class<?> iFace) throws SQLException {
        return connection.isWrapperFor(iFace);
    }

    public void setSchema(String schema) throws SQLException {
        connection.setSchema(schema);
    }

    public String getSchema() throws SQLException {
        return connection.getSchema();
    }

    public void abort(Executor executor) throws SQLException {
        connection.abort(executor);
    }

    public void setNetworkTimeout(Executor executor, int milliseconds) throws SQLException {
        connection.setNetworkTimeout(executor, milliseconds);
    }

    public int getNetworkTimeout() throws SQLException {
        return connection.getNetworkTimeout();
    }

}
